/* eslint-disable no-magic-numbers */
import { SIZE_OF_EMPTY_ARRAY } from '@/constants/common';
import { bioEntityResponseFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { reactionsFixture } from '@/models/fixtures/reactionFixture';
import {
  ELEMENT_SEARCH_RESULT_MOCK_ALIAS,
  ELEMENT_SEARCH_RESULT_MOCK_REACTION,
} from '@/models/mocks/elementSearchResultMock';
import { apiPath } from '@/redux/apiPath';
import { mockNetworkNewAPIResponse, mockNetworkResponse } from '@/utils/mockNetworkResponse';
import { getReduxStoreActionsListener } from '@/utils/testing/getReduxStoreActionsListener';
import { waitFor } from '@testing-library/react';
import { HttpStatusCode } from 'axios';
import { MapBrowserEvent } from 'ol';
import * as onMapSingleClickUtils from './onMapSingleClick';
import { handleAliasResults, handleReactionResults } from './onMapSingleClick';

const { onMapSingleClick } = onMapSingleClickUtils;
const mockedAxiosOldClient = mockNetworkResponse();
const mockedAxiosNewClient = mockNetworkNewAPIResponse();

const getEvent = (coordinate: MapBrowserEvent<UIEvent>['coordinate']): MapBrowserEvent<UIEvent> =>
  ({
    coordinate,
  }) as unknown as MapBrowserEvent<UIEvent>;

describe('onMapSingleClick - util', () => {
  beforeEach(() => {
    jest.clearAllMocks();
    jest.resetAllMocks();
  });

  describe('when searchResults are undefined', () => {
    const { store } = getReduxStoreActionsListener();
    const { dispatch } = store;
    const modelId = 1000;
    const mapSize = {
      width: 90,
      height: 90,
      tileSize: 256,
      minZoom: 2,
      maxZoom: 9,
    };
    const handler = onMapSingleClick(mapSize, modelId, dispatch);
    const coordinate = [90, 90];
    const point = { x: 180.0008084837557, y: 179.99919151624428 };
    const event = getEvent(coordinate);

    mockedAxiosOldClient
      .onGet(apiPath.getSingleBioEntityContentsStringWithCoordinates(point, modelId))
      .reply(HttpStatusCode.Ok, undefined);

    it('does not fire search result action', async () => {
      await handler(event);
      const actions = store.getActions();
      expect(actions.length).toBe(SIZE_OF_EMPTY_ARRAY);
    });
  });

  describe('when searchResults are empty', () => {
    const { store } = getReduxStoreActionsListener();
    const { dispatch } = store;

    const modelId = 1000;
    const mapSize = {
      width: 180,
      height: 180,
      tileSize: 256,
      minZoom: 2,
      maxZoom: 9,
    };

    const handler = onMapSingleClick(mapSize, modelId, dispatch);
    const coordinate = [180, 180];
    const point = { x: 360.0032339350228, y: 359.9967660649771 };
    const event = getEvent(coordinate);

    mockedAxiosOldClient
      .onGet(apiPath.getSingleBioEntityContentsStringWithCoordinates(point, modelId))
      .reply(HttpStatusCode.Ok, []);

    it('does not fire search result action', async () => {
      await handler(event);
      const actions = store.getActions();
      expect(actions.length).toBe(SIZE_OF_EMPTY_ARRAY);
    });
  });

  describe('when searchResults are valid', () => {
    describe('when results type is ALIAS', () => {
      const { store } = getReduxStoreActionsListener();
      const { dispatch } = store;
      const { modelId } = ELEMENT_SEARCH_RESULT_MOCK_ALIAS;
      const mapSize = {
        width: 270,
        height: 270,
        tileSize: 256,
        minZoom: 2,
        maxZoom: 9,
      };
      const coordinate = [270, 270];
      const point = { x: 540.0072763538013, y: 539.9927236461986 };
      const event = getEvent(coordinate);

      mockedAxiosOldClient
        .onGet(apiPath.getSingleBioEntityContentsStringWithCoordinates(point, modelId))
        .reply(HttpStatusCode.Ok, [ELEMENT_SEARCH_RESULT_MOCK_ALIAS]);

      beforeAll(async () => {
        const handler = onMapSingleClick(mapSize, modelId, dispatch);
        await handler(event);
      });

      it('does fire search result action - getBioEntity', async () => {
        const actions = store.getActions();
        expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
        expect(actions[0].type).toEqual('project/getBioEntityContents/pending');
      });

      it('does NOT fire search result action - getReactionsByIds', async () => {
        const actions = store.getActions();
        expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
        expect(actions[0].type).not.toEqual('reactions/getByIds/pending');
      });
    });

    describe('when results type is REACTION', () => {
      const { store } = getReduxStoreActionsListener();
      const { dispatch } = store;
      const { modelId } = ELEMENT_SEARCH_RESULT_MOCK_REACTION;
      const mapSize = {
        width: 0,
        height: 0,
        tileSize: 256,
        minZoom: 2,
        maxZoom: 9,
      };
      const coordinate = [0, 0];
      const point = {
        x: 0,
        y: 0,
      };
      const event = getEvent(coordinate);

      mockedAxiosOldClient
        .onGet(apiPath.getSingleBioEntityContentsStringWithCoordinates(point, modelId))
        .reply(HttpStatusCode.Ok, [ELEMENT_SEARCH_RESULT_MOCK_REACTION]);

      beforeAll(async () => {
        const handler = onMapSingleClick(mapSize, modelId, dispatch);
        await handler(event);
      });

      it('does NOT fire search result action - getBioEntity', async () => {
        const actions = store.getActions();
        expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
        expect(actions[0].type).not.toEqual('project/getBioEntityContents/pending');
      });

      it('does fire search result action - getReactionsByIds', async () => {
        const actions = store.getActions();
        expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
        expect(actions[0].type).toEqual('reactions/getByIds/pending');
      });
    });
  });
});

describe('handleAliasResults - util', () => {
  const { store } = getReduxStoreActionsListener();
  const { dispatch } = store;

  mockedAxiosOldClient
    .onGet(
      apiPath.getBioEntityContentsStringWithQuery(ELEMENT_SEARCH_RESULT_MOCK_ALIAS.id.toString(), {
        perfectMatch: true,
      }),
    )
    .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

  beforeAll(async () => {
    handleAliasResults(dispatch)(ELEMENT_SEARCH_RESULT_MOCK_ALIAS);
  });

  it('should run getBioEntityAction', async () => {
    await waitFor(() => {
      const actions = store.getActions();
      expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
      expect(actions[0].type).toEqual('project/getBioEntityContents/pending');
    });
  });
});

describe('handleReactionResults - util', () => {
  const { store } = getReduxStoreActionsListener();
  const { dispatch } = store;

  mockedAxiosNewClient
    .onGet(
      apiPath.getBioEntityContentsStringWithQuery(
        ELEMENT_SEARCH_RESULT_MOCK_REACTION.id.toString(),
        {
          perfectMatch: true,
        },
      ),
    )
    .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

  mockedAxiosOldClient
    .onGet(apiPath.getReactionsWithIds([ELEMENT_SEARCH_RESULT_MOCK_REACTION.id]))
    .reply(HttpStatusCode.Ok, reactionsFixture);

  beforeAll(async () => {
    handleReactionResults(dispatch)(ELEMENT_SEARCH_RESULT_MOCK_REACTION);
  });

  it('should run getReactionsByIds as first action', () => {
    const actions = store.getActions();
    expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
    expect(actions[0].type).toEqual('reactions/getByIds/pending');
    expect(actions[1].type).toEqual('reactions/getByIds/fulfilled');
  });

  it('should run setBioEntityContent to empty array as second action', () => {
    const actions = store.getActions();
    expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
    expect(actions[2].type).toEqual('bioEntityContents/setBioEntityContent');
  });

  it('should run getBioEntity as third action', () => {
    const actions = store.getActions();
    expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
    expect(actions[3].type).toEqual('project/getBioEntityContents/pending');
  });
});
